'use strict';

var _ = require('lodash');
var React = require('react');

var _require = require('react');

var PropTypes = _require.PropTypes;

var _require2 = require('immutable');

var List = _require2.List;

var EntityList = require('./EntityList');
var EntityView = require('./EntityView');
var EntityListItem = require('./EntityListItem');

var _require3 = require('react-bootstrap');

var Grid = _require3.Grid;
var Row = _require3.Row;
var Col = _require3.Col;
var Button = _require3.Button;

var NavigationManager = require('./NavigationManager');

var _require4 = require('./Util');

var pluralize = _require4.pluralize;


var EntityListWithView = React.createClass({
    displayName: 'EntityListWithView',

    render: function render() {
        var p = this.props;
        var entityManager = p.entityManager;
        var selectedId = p.selectedId;
        var entity = selectedId === "new" ? entityManager.newInstance() : entityManager.get(selectedId);
        var displayItemFn = function displayItemFn(item) {
            return React.createElement(EntityListItem, { item: item });
        };
        return React.createElement(
            Grid,
            null,
            React.createElement(
                Row,
                null,
                React.createElement(
                    Col,
                    { xs: 12, md: 3 },
                    React.createElement(
                        'h3',
                        null,
                        pluralize(entityManager.typeName)
                    ),
                    React.createElement(
                        'p',
                        null,
                        this.props.onNew || this.props.navigationManager ? React.createElement(
                            Button,
                            { onClick: this.newObject, bsSize: 'xsmall' },
                            'New'
                        ) : ''
                    ),
                    React.createElement(EntityList, { items: entityManager.choiceList(), selectedItemId: selectedId, onSelect: this.select, displayItem: displayItemFn })
                ),
                React.createElement(
                    Col,
                    { xs: 12, md: 9 },
                    this.entityView(entity)
                )
            )
        );
    },

    entityView: function entityView(entity) {
        if (!entity) {
            return "";
        }

        var viewType = this.props.entityViewType || EntityView;
        return React.createElement(viewType, { entity: entity, onSave: this.saveEntity });
    },

    select: function select(entity) {
        if (this.props.onSelect) {
            this.props.onSelect(entity.id);
        } else {
            this.props.navigationManager.navigate(entity.id);
        }
    },

    saveEntity: function saveEntity(entity) {
        var newEntity = !entity.id;
        var savedEntity = this.props.entityManager.save(entity);
        if (newEntity) {
            this.select(savedEntity);
        }
    },

    newObject: function newObject() {
        if (this.props.onNew) {
            this.props.onNew();
        } else {
            this.props.navigationManager.navigateNew();
        }
    }
});

EntityListWithView.propTypes = {
    entityManager: PropTypes.object.isRequired,
    navigationManager: PropTypes.instanceOf(NavigationManager).isRequired,
    entityViewType: PropTypes.func,
    selectedId: PropTypes.string,
    onSelect: PropTypes.func,
    onNew: PropTypes.func
};

module.exports = EntityListWithView;