'use strict';

var _ = require('lodash');
var React = require('react');

var _require = require('react');

var PropTypes = _require.PropTypes;

var _require2 = require('immutable');

var List = _require2.List;

var _require3 = require('react-router-component');

var Link = _require3.Link;

var _require4 = require('./Util');

var parseProp = _require4.parseProp;

var DisplayItem = require('./DisplayItem');

var EntityTableRowGroup = React.createClass({
    displayName: 'EntityTableRowGroup',


    render: function render() {
        var _this = this;

        return React.createElement(
            'tbody',
            { className: this.props.className },
            this.props.items.map(function (item, index) {
                return _this.dataRow(item, index);
            })
        );
    },

    entityDescriptor: function entityDescriptor() {
        var items = this.props.items;
        var entityDescriptor = this.props.entityDescriptor || items.get(0) && items.get(0).constructor.entityDescriptor;
        if (!entityDescriptor) throw new Error('EntityDescriptor required'); //TODO show no data message if no items and no entity descriptor
        return entityDescriptor;
    },

    propertiesToShow: function propertiesToShow() {
        return (this.props.propertiesToShow || this.entityDescriptor().displayProperties).map(parseProp);
    },

    dataCell: function dataCell(propInfo, item, index) {
        var name = propInfo.name;
        var itemLink = propInfo.itemLink;

        if (name) {
            var propDesc = this.entityDescriptor().propertyDescriptor(name);
            var typeClassName = propDesc.type.name.toLowerCase();
            var value = item[name];
            var displayItem = React.createElement(DisplayItem, { propDesc: propDesc, value: value });
            var content = itemLink ? this.itemLink(item, itemLink, displayItem) : displayItem;
            return React.createElement(
                'td',
                { key: name, className: typeClassName },
                content
            );
        } else {
            return React.createElement('td', { key: "empty" + index, className: 'empty' });
        }
    },

    itemLink: function itemLink(item, itemLinkPropertyName, displayItem) {
        if (typeof itemLinkPropertyName === 'string') {
            var linkPropertyDesc = this.entityDescriptor().propertyDescriptor(itemLinkPropertyName);
            var linkPropertyValue = item[itemLinkPropertyName];
            var navigationManager = this.context.getNavigationManager(linkPropertyDesc.itemType);
            return React.createElement(
                Link,
                { href: navigationManager.pathTo(linkPropertyValue) },
                displayItem
            );
        } else {
            var _navigationManager = this.context.getNavigationManager(item.constructor);
            return React.createElement(
                Link,
                { href: _navigationManager.pathTo(item) },
                displayItem
            );
        }
    },

    dataRow: function dataRow(item, index) {
        var _this2 = this;

        return React.createElement(
            'tr',
            { key: item.id || index },
            this.propertiesToShow().map(function (prop, index) {
                return _this2.dataCell(prop, item, index);
            })
        );
    }
});

EntityTableRowGroup.propTypes = {
    items: PropTypes.instanceOf(List).isRequired,
    entityDescriptor: PropTypes.object,
    propertiesToShow: PropTypes.arrayOf(PropTypes.string)
};

EntityTableRowGroup.contextTypes = {
    getNavigationManager: PropTypes.func
};

module.exports = EntityTableRowGroup;