'use strict';

var React = require('react');

var _require = require('react');

var PropTypes = _require.PropTypes;

var _require2 = require('react-bootstrap');

var FormGroup = _require2.FormGroup;
var ControlLabel = _require2.ControlLabel;
var FormControl = _require2.FormControl;
var HelpBlock = _require2.HelpBlock;

var _require3 = require('immutable');

var List = _require3.List;

var moment = require('moment');
var EntityListItem = require('./EntityListItem');
var EntityViewFn = function EntityViewFn() {
    return require('./EntityView');
};
var SimpleView = require('./SimpleView');
var EntityListEditable = require('./EntityListEditable');
var SimpleListEditable = require('./SimpleListEditable');
var DisplayItem = require('./DisplayItem');
var DateTimeField = require('react-bootstrap-datetimepicker');

var _require4 = require('lsd-metadata');

var Types = _require4.Types;
var Reference = _require4.Reference;


var FormItem = React.createClass({
    displayName: 'FormItem',

    getInitialState: function getInitialState() {
        var fieldId = 'id_' + (Math.floor(Math.random() * 1000000) + 1);
        return { value: this.props.value, fieldId: fieldId };
    },
    handleChange: function handleChange(event) {
        var valueStr = event.target.value;
        var value = valueStr === '' ? null : valueStr;
        this.setState({ value: value });
        if (this.props.onChange) {
            this.props.onChange(value);
        }
    },

    handleNumberChange: function handleNumberChange(event) {
        var valueStr = event.target.value;
        var value = valueStr === '' ? null : parseFloat(valueStr);
        this.setState({ value: value });
        if (this.props.onChange) {
            this.props.onChange(value);
        }
    },

    handleDateChange: function handleDateChange(valueStr) {
        var value = valueStr === 'Invalid date' ? null : new Date(parseInt(valueStr));
        this.setState({ value: value });
        if (this.props.onChange) {
            this.props.onChange(value);
        }
    },

    handleListChange: function handleListChange(value) {
        this.setState({ value: value });
        if (this.props.onChange) {
            this.props.onChange(value);
        }
    },

    handleSelectChange: function handleSelectChange(event) {
        var optionValue = event.target.value;
        var value = this.props.type.values().find(function (x) {
            return x.name === optionValue;
        }) || null;
        this.setState({ value: value });
        if (this.props.onChange) {
            this.props.onChange(value);
        }
    },

    handleRefChange: function handleRefChange(event) {
        var value = event.target.value;
        this.setState({ value: value });
        if (this.props.onChange) {
            this.props.onChange(value);
        }
    },

    render: function render() {
        var validationState = this.props.error ? "error" : null;
        var label = this.props.label ? React.createElement(
            ControlLabel,
            null,
            this.props.label
        ) : "";
        var error = this.props.error ? React.createElement(
            HelpBlock,
            null,
            this.props.error
        ) : this.props.help ? React.createElement(
            HelpBlock,
            null,
            this.props.help
        ) : '';

        return React.createElement(
            FormGroup,
            { controlId: this.state.fieldId, validationState: validationState },
            label,
            this.formControl(),
            React.createElement(FormControl.Feedback, null),
            error
        );
    },

    getDefaultProps: function getDefaultProps() {
        return {
            value: ''
        };
    },

    componentWillReceiveProps: function componentWillReceiveProps(props) {
        this.setState({ value: props.value });
    },

    formControl: function formControl() {
        var EntityView = EntityViewFn();
        var _props = this.props;
        var propDesc = _props.propDesc;
        var type = _props.type;
        var editable = _props.editable;
        var placeholder = _props.placeholder;
        var viewElement = _props.viewElement;

        var value = this.state.value;
        var valueOrBlank = this.state.value === undefined || this.state.value === null ? "" : this.state.value;

        if (viewElement) {
            return React.cloneElement(viewElement, { items: value, entityDescriptor: propDesc.itemType.entityDescriptor });
        }

        if (!editable) {
            return React.createElement(
                FormControl.Static,
                null,
                React.createElement(DisplayItem, { propDesc: propDesc, value: valueOrBlank })
            );
        }

        if (type === String) {
            return React.createElement(FormControl, { type: 'text', value: valueOrBlank, placeholder: placeholder, onChange: this.handleChange, autoFocus: this.props.autoFocus });
        }
        if (type === Number) {
            return React.createElement(FormControl, { type: 'text', value: valueOrBlank, placeholder: placeholder, onChange: this.handleNumberChange, autoFocus: this.props.autoFocus });
        }
        if (type === Date) {
            var displayFormat = "DD MMM YY";
            return React.createElement(DateTimeField, { mode: 'date', inputFormat: displayFormat, onChange: this.handleDateChange,
                dateTime: value ? value.getTime() : undefined });
        }
        if (type === List) {
            var displayItemFn = function displayItemFn(item) {
                return React.createElement(EntityListItem, { item: item });
            };

            if (Types.isEntity(propDesc.itemType)) {
                var editItemFn = function editItemFn(item, onSave) {
                    return React.createElement(EntityView, { entity: item, onSave: onSave });
                };
                return React.createElement(EntityListEditable, { items: value, entityManager: this.context.getEntityManager(propDesc.itemType), displayItem: displayItemFn, editItem: editItemFn, onChangeList: this.handleListChange });
            } else {
                return React.createElement(SimpleListEditable, { items: value, itemType: propDesc.itemType, onChangeList: this.handleListChange });
            }
        }
        if (Types.isSameType(type, Reference)) {
            var entities = this.context.getEntityManager(propDesc.itemType).choiceList();
            var optionList = entities.map(function (o) {
                return { value: o.id, name: o.shortSummary };
            });
            return React.createElement(
                FormControl,
                { componentClass: 'select', value: value, onChange: this.handleRefChange },
                React.createElement(
                    'option',
                    { value: '' },
                    placeholder || "Not selected"
                ),
                optionList.map(function (op) {
                    return React.createElement(
                        'option',
                        { key: op.value, value: op.value },
                        op.name
                    );
                })
            );
        }
        if (type.values) {
            var _optionList = type.values().map(function (o) {
                return { value: o.name, name: o.label };
            });
            return React.createElement(
                FormControl,
                { componentClass: 'select', value: value, onChange: this.handleSelectChange },
                React.createElement(
                    'option',
                    { value: '' },
                    placeholder || "Not selected"
                ),
                _optionList.map(function (op) {
                    return React.createElement(
                        'option',
                        { key: op.value, value: op.value },
                        op.name
                    );
                })
            );
        }

        return 'Type: ' + type + ' Value: ' + value;
    }
});

FormItem.propTypes = {
    viewElement: PropTypes.element,
    propDesc: PropTypes.object,
    value: PropTypes.any,
    label: PropTypes.string,
    placeholder: PropTypes.string,
    help: PropTypes.string,
    error: PropTypes.string,
    editable: PropTypes.bool,
    onChange: PropTypes.func,
    autoFocus: PropTypes.bool
};

FormItem.contextTypes = {
    getEntityManager: PropTypes.func
};

module.exports = FormItem;